%function vals=dphi(targ,src,nsrc,alpha,E)
% returns the normal derivative with respect to the source, dphi/dny of the
%  fundamental solution, given the source normal ny.
%INPUTS:
%   targ: matrix of targets: complex coordinates
%   src: matrix of sources, one for each target: complex coordinates
%       or single source for all targets
%   nsrc: matrix of normal vectors to each source point
%   alpha, E: parameters of the PDE
%OUTPUTS:
%   vals: dphi/dny at each target point
%BJN 5/11/12. Barnett 9/17/14 changed for single call

function vals=dphi(targ,src,nsrc,alpha,E)
vals=zeros(size(targ))+1i*zeros(size(targ));

if alpha==0 && E==0   %Laplace
  dist=targ-src;
  vals=1/(2*pi)*(real(nsrc).*real(dist)+imag(nsrc).*imag(dist))./(abs(dist)).^2;
  
elseif alpha==0 && E~=0            %Helmholtz
  k=sqrt(E);
  dist=targ-src;
  vals=1i*k/4*(real(nsrc).*real(dist)+imag(nsrc).*imag(dist))./...
       abs(dist).*besselh(1,k*abs(dist));

elseif alpha~=0              %Linear Helmholtz (careful w/ transposes)
  [u u1 u2] = lhelmfs([real(targ(:))';imag(targ(:))'],[real(src(:))';imag(src(:))'],...
                      alpha,E);
  vals = reshape(u1(:).*real(nsrc(:))+u2(:).*imag(nsrc(:)), size(targ));
  %    vals=vals*alpha^(2/3);
end