% convergence of BIE (tests kernels & quadrature)
% Barnett 9/19/14 based on test_GRF.m

%alpha=0; E=0;  % Laplace
%alpha=0; E=1;  % Helmholtz
alpha=1; E=10;    % linear Helmholtz
k=sqrt(E);
%a = [1 0 0 0 0 0 0]; m = 60;  % 1e-12 for circle
a=5*[1 0 0 0.3 0 0 0]; % 1e-11, boundary Fourier shape coefficients
inside = @(z) abs(z)<sqrt(sum(paramt(angle(z),a).^2));

np = 100;   % int pts to eval at for convergence
[z,dzdx,n,kap]=param(np,a); %figure; plot(z(1,:),z(2,:),'k.'); axis equal tight; drawnow;
zp = (z(1,:)+1i*z(2,:)).*(0.8*sqrt(rand(1,np))); % C# targ list, keep away from bdry

ms = 40:20:320; es = nan(2,numel(ms)); ups = nan(np,numel(ms));  % grf err & u vals
for i=1:numel(ms), m=ms(i)  % .........
  tic; [D, S, z, dzdx, w, n]=kerDS(m,E,alpha,a,'a'); % fill matrices

if E>0 && alpha==0, %-------helmholtz solution
    u=@(x) sin(k*x(1,:));
    ux=@(x) k*cos(k*x(1,:));
    uy=@(x) zeros(size(x(1,:)));
elseif E==0 && alpha==0
    %------ Laplace solution
    u=@(x) x(1,:).*x(2,:);
    ux=@(x) x(2,:);
    uy=@(x) x(1,:);
else % --------- lin Helm ----
    u=@(x) sin(k*(x(1,:))).*airy(0,-nthroot(alpha,3)*x(2,:));
    ux=@(x) k*cos(k*(x(1,:))).*airy(0,-nthroot(alpha,3)*x(2,:));
    uy=@(x) -nthroot(alpha,3)*sin(k*(x(1,:))).*airy(1,-nthroot(alpha,3)*x(2,:));
%     src=[2;0];
%     u=@(x) lhelmfs(x,src,alpha,E,1);
end

b=zeros(size(z(1,:)))';  % knowns bdry data
for j=1:numel(z(1,:)), b(j)=u(z(:,j)); end
bn=zeros(size(z(1,:)'));
for j=1:numel(z(1,:)), bn(j)=ux(z(:,j))*n(1,j)+uy(z(:,j))*n(2,j); end

tau = (D-eye(m)/2)\b; toc  % solve BIE

es(1,i) = norm(S*bn-(D+eye(m)/2)*b);
fprintf('||S un - (D+1/2) u||_l2 = %.3g\n',es(1,i)) % interior GRF w/ JR

p = 0.2+0.3i; % pt to evaluate soln...
zc=z(1,:)+1i*z(2,:); nc=n(1,:)+1i*n(2,:);
f=sum(dphi(p+0*zc,zc,nc,alpha,E).*tau.'.*dzdx.*w);
es(2,i) = f-u([real(p); imag(p)]);
fprintf('err @ p: %.13g\n',abs(es(2,i)))

Ap = dphi(repmat(zp.',[1 m]),repmat(zc,[np 1]),repmat(nc,[np 1]),alpha,E);
ups(:,i) = Ap*(tau.*dzdx.'.*w.');
end     % ..........

%figure; plot(eig(D-eye(m)/2),'+'); axis equal  % check not on-resonance
upse = u([real(zp); imag(zp)]).';

figure;
semilogy(ms,max(abs(ups-repmat(upse,[1 numel(ms)]))),'s-'); hold on;
semilogy(ms,abs(es(1,:)),'-+'); xlabel('N'); ylabel('error');
plot(ms,4e28./ms.^16.*log(ms), 'r--')   % convergence rate
legend({'$$u$$ max error over interior points','GRF boundary data $$l_2$$ error','$$N^{-16} \log N$$'},'interpreter','latex')
axis([40 320 1e-12 1]);
set(gcf,'paperposition',[0 0 5 4]);
print -depsc2 ../../bie_paper/figs/Airyconv.eps

%figure; plot(z(1,:),z(2,:),'k.'); axis equal tight; hold on; plot(zp,'.'); % shape

% GRF is around 10th ord rate

if 0, % evaluate on grid ------------- u plot
g = 5*(-1.4:0.01:1.4); [xx yy] = meshgrid(g); zz = xx(:)' + 1i*yy(:)';
ii = inside(zz);  % keep just pts inside
zg = zz(ii); n = numel(zg)
'u eval...', tic
Ag = dphi(repmat(zg.',[1 m]),repmat(zc,[n 1]),repmat(nc,[n 1]),alpha,E);
toc
ug = nan*xx; ug(ii) = Ag*(tau.*dzdx.'.*w.');
figure; imagesc(g,g,real(ug),'alphadata',~isnan(ug)); hold on; plot(z(1,:),z(2,:),'k.-');
axis xy equal tight; colorbar; caxis(.5*[-1 1]); axis([-5 7 -6.5 6.5]);
set(gcf,'paperposition',[0 0 5 4]); print -depsc2 ../../bie_paper/figs/Airyu_m260.eps
end

%clear Ag Ap 
%save Airyconv.mat
