% kernel function: fills kernel for laplace or helm eqn
%               also gives nodes and weights
% inputs k : wavenumber
%        m : number of nodes
%        a : fourier coefficients
%
% outputs ker: kernel matrix
%         [z dzdx, w, n] : parameterization


function [A, z, dzdx, w, n]=ker(m,E,alpha,a,quad)


%first, set up nodes and weights

[z,dzdx,n,kap]=param(m,a); %parameterizes equation

w=2*pi/m; %periodic trapezoid weights


%now to set up kernel matrix
A=zeros(m,m); %initalizes kernel matrix

%puts in complex form
zc=z(1,:)'+1i*(z(2,:)');
nc=n(1,:)'+1i*(n(2,:)');

for j=1:m
    A(:,j)= dphi(zc,zc(j),nc(j),alpha,E)*dzdx(j);
end



if alpha==0
    for i=1:m; %does diagonal entries- same for laplace or helm eqns
        A(i,i)=-kap(i)/(4*pi)*dzdx(i);
    end
end


% apply weights...
if nargin~=5, quad='t'; end % trap rule default

if quad=='t', A = A*w;
elseif quad=='k' %KR modifications
    [s w] = kapurtrap(m+1, 10); % Zydrunas-supplied K-R weights
    w = w*2*pi;    % change interval of s to [0,2pi]
    A = circulant(w(1:end-1)) .* A;
elseif quad=='a'  % Alpert
    %A(diagind(A)) = 0;
    A = w*A;
    A = A - diag(diag(A));  % inefficient
    o = []; o.ord = 16;
    A = alpertizematrix(A, @(t) paramt(t,a), @(t,s,n) dphi(t,s,n,alpha,E), o);
end

w=ones(1,m)*2*pi/m;
