%[D, S, z, dzdx, w, n]=kerDS(m,E,alpha,a,quad)
%returns double and single layer kernel along with parameterization
%information
%INPUTS:
% m: number of boundary points
% E, alpha: parameters of the PDE
% a: boundary coefficient vector
% quad: quadrature scheme
%   't' (default) trapezoid rule
%   'k' kapur-rohklin
%   'a' alpert
%OUTPUTS:
%   D: double layer kernel
%   S: single layer kernel
%   z: boundary point locations
%   dzdx: parameterization speed through boundary points
%   w: weights of points
%   n: normals at points
%BJN 5/15, edited Barnett 9/17/14 to make single MEX call in Alpert
% Barnett 9/17/14/ signs all fixed.

function [D, S, z, dzdx, w, n]=kerDS(m,E,alpha,a,quad)

addpath('../'); % for lhelmfs
%first, set up nodes and weights

[z,dzdx,n,kap]=param(m,a); %parameterizes equation

w=2*pi/m; %periodic trapezoid weights (const)

%now to set up kernel matrices
D=zeros(m,m); %initalizes D kernel matrix
S=zeros(m,m); %initalizes S kernel matrix

if alpha==0
    %puts in complex form
    zc=z(1,:)'+1i*(z(2,:)');
    nc=n(1,:)'+1i*(n(2,:)');
    for j=1:m            % loop over sources
        D(:,j)= dphi(zc,zc(j),nc(j),alpha,E)*dzdx(j);
        S(:,j)= phi(zc,zc(j),alpha,E)*dzdx(j);
    end
else             % linear Helmholtz (faster to get both S,D together)
    for j=1:m       % loop over sources
        [u ux0 uy0]=lhelmfs(z,z(:,j),alpha,E);  % alex 9/12/14
        D(:,j)=((ux0.')*n(1,j)+(uy0.')*n(2,j))*dzdx(j);
        S(:,j)=u.'*dzdx(j);
    end
    %S=S*alpha^(2/3); D=D*alpha^(2/3); %normalizes
end
%sprintf('fill bare D,S size %d: %.3g sec\n',m,toc)
    
% apply weights...
if nargin~=5, quad='t'; end % trap rule default

if quad=='t'
  for i=1:m; % diagonal entries, exact for D for Laplace only
    D(i,i)=-kap(i)/(4*pi)*dzdx(i); % (3rd-ord acc for Helm or lin helm)
    S(i,i)=0;   % dummy to prevent Inf, less than 1st order acc.
  end
  D=D*w; S = S*w;
elseif quad=='k' %KR modifications
    [s w] = kapurtrap(m+1, 10); % Zydrunas-supplied K-R weights
    w = w*2*pi;    % change interval of s to [0,2pi]
    D(diagind(D)) = 0; S(diagind(S)) = 0; % kill diag (otherwise nan)
    D = circulant(w(1:end-1)) .* D;
    S = circulant(w(1:end-1)) .* S;
elseif quad=='a'  % Alpert
    D = w*D; S=w*S; D(diagind(D)) = 0; S(diagind(S)) = 0; % kill diag
    o = []; o.ord = 16;        % alpert order
    D = alpertizematrix(D, @(t) paramt(t,a), @(t,s,n) dphi(t,s,n,alpha,E), o);
    S = alpertizematrix(S, @(t) paramt(t,a), @(t,s,n) phi(t,s,alpha,E), o);
    %fprintf('alpertize D,S size %d: %.3g sec\n',m,toc)
end

w=ones(1,m)*2*pi/m;
