%function vals=phi(targ,src,alpha,E)
% returns the fundamental solution value.
%INPUTS:
%   targ: matrix of targets: complex coordinates
%   src: matrix of sources, one for each target: complex coordinates
%       or single source for all targets
%   alpha, E: parameters of the PDE
%OUTPUTS:
%   vals: phi at each target point
%BJN 5/11/12. Barnett changed for single call, 9/17/14

function vals=phi(targ,src,alpha,E)
vals=zeros(size(targ))+1i*zeros(size(targ));

if alpha==0 && E==0           % Laplace
  dist=targ-src;
  vals=-1/2/pi*log(abs(dist));
  return;

elseif alpha==0 && E~=0       %Helmholtz
  k=sqrt(E);
  dist=targ-src;
  vals=1i/4*besselh(0,k*abs(dist));
  return;
  
elseif alpha~=0           %Linear Helmholtz (careful w/ transposes)
  vals = lhelmfs([real(targ(:))';imag(targ(:))'],[real(src(:))';imag(src(:))'],alpha,E);
  vals = reshape(vals,size(targ));
  %vals=vals*alpha^(2/3);
end