% timing (& # quadr nodes) vs E figure
clear;
Es = [-100 -30 -10 -3 -1 1 3 10 30 1e2 3e2 1e3 3e3 1e4];
n = 1e4;
r = 10.^(-1 + 5*rand(1,n)); % radii log distributed, far cases
rc = 10.^(-4 + 3*rand(1,n)); % radii log distributed, close cases
t = rand(1,n)*2*pi;          % angles uniform
y = [0;0]; x = [r.*cos(t); r.*sin(t)]; xc = [rc.*cos(t); rc.*sin(t)];
ts = nan(4,numel(Es));  % timing data: 1st row vals 2nd row +derivs, 3+4 same close
for i=1:numel(Es), fprintf('E = %.3g\n',Es(i))
  tic; [u u1 u2 info] = lhelmfs(x,y,[],Es(i),0); ts(1,i) = toc; ns(1,i) = info.nquad;
  tic; [u u1 u2 info] = lhelmfs(x,y,[],Es(i),1); ts(2,i) = toc; ns(2,i) = info.nquad;
  tic; [u u1 u2 info] = lhelmfs(xc,y,[],Es(i),0); ts(3,i) = toc; ns(3,i) = info.nquad;
  tic; [u u1 u2 info] = lhelmfs(xc,y,[],Es(i),1); ts(4,i) = toc; ns(4,i) = info.nquad;
end
R = n./ts; % rates in evals/sec
N = ns/n;  % mean # quadr pts per eval
%save timevsE.mat
% load timevsE.mat
figure;
subplot(1,2,1);
semilogx(Es,N(1,:),'b+-',Es,N(2,:),'b+--',Es,N(3,:),'g+-',Es,N(4,:),'g+--');
hold on;
semilogx(-Es,N(1,:),'bs-',-Es,N(2,:),'bs--',-Es,N(3,:),'gs-',-Es,N(4,:),'gs--');
legend('\Phi','\Phi,\Phi_1,\Phi_2','near \Phi','near \Phi,\Phi_1,\Phi_2','location','east');
xlabel('|E|'); ylabel('mean nodes per eval');
text(1e-1,720,'(a)');
subplot(1,2,2);
semilogx(Es,R(1,:),'b+-',Es,R(2,:),'b+--',Es,R(3,:),'g+-',Es,R(4,:),'g+--');
hold on;
semilogx(-Es,R(1,:),'bs-',-Es,R(2,:),'bs--',-Es,R(3,:),'gs-',-Es,R(4,:),'gs--');
v = axis; v(3)=0; axis(v);
legend('\Phi','\Phi,\Phi_1,\Phi_2','near \Phi','near \Phi,\Phi_1,\Phi_2','location','southeast');
xlabel('|E|'); ylabel('evals/sec');
text(1e-1,1.23e5,'(b)');
set(gcf,'paperposition',[0 0 9 3.5]);
%print -depsc2 ../bie_paper/figs/timevsE.eps
