% Convergence and other tests for for lhelmfs MEX file
% Barnett 9/13/14
clear; format long g

if 0   % Basic tests of calling args when singular...
  x = [1;2]; E = 10; 'should be inf', lhelmfs(x,x,[],E)
  'should all be inf', [u u1 u2]=lhelmfs(x,x,[],E)
  'u only should be inf', [u u1 u2]=lhelmfs(x,x,[],E,0)
  x = [1 2; 3 4]; 'us only should be inf', [u u1 u2]=lhelmfs(x,x,[],E,0)
  o.meth=1; o.h = 0; [u u1 u2]=lhelmfs(x,x,[],E,[],o);
end

if 1 %%%%% Check convergence of two methods at one pt...
%E = 20; y = [0;0]; x = [1;2];  % params: y=src, x=targ
%E = 1e2; y = [0;0]; x = [1;-E-0.01];  % params: y=src, x=targ
%E = 3; y = [0;0]; x = [1e-3;0];
%E = 0; y = [0;0]; x = [1e-4;1e-3];
%E = 100; y = [0;0]; x = [1976.59;9768.66]; % coalescing saddle, region A
%E = -10; y = [0;0]; x = [0.275055;-7.20553];
E = -100; y=[0;0]; x = [0.000118063;-6.7417e-05];
  
o.meth = 0; Ns = 50:50:200; for i=1:numel(Ns),
  o.stdquad = Ns(i); [u u1 u2] = lhelmfs(x,y,[],E,[],o);
  fprintf('stdquad=%d: \t u = %.16g + \t %.16g i\n',Ns(i),real(u),imag(u)), end
  
o.meth = 1; hs = 0.5:-0.05:0.2; for i=1:numel(hs),
  o.h = -hs(i); [u u1 u2] = lhelmfs(x,y,[],E,[],o); % -h makes all params conv w/ h
  fprintf('h=%.3f: \t u = %.16g + \t %.16g i\n',hs(i),real(u),imag(u)), end

o = []; o.meth=0; [ue u1e u2e] = lhelmfs(x,y,[],E,[],o); % 'exact'
[u u1 u2] = lhelmfs(x,y,[],E);  % default call (meth=1, h standard)
fprintf('compare default vs meth0:  rel err u = %.3g,   rel err grad u = %.3g\n',...
        abs(u-ue)/abs(u), norm([u1,u2]-[u1e,u2e])/norm([u1 u2]))

% Check the nodes and summand size from the default call at the above point...
system('rm -f nodes.dat ctr.dat');
o.gamout = 1; o.meth = 1; o.h = 0.3; [u u1 u2 i] = lhelmfs(x,y,[],E,[],o);
fprintf('#quad nodes = %d\n',i.nquad);
g = textread('ctr.dat'); g = g(:,1)+1i*g(:,2);  % full contour in complex numbers
gj = textread('nodes.dat');
if ~isempty(gj)
  sj = gj(:,3);          % abs summands
  gj = gj(:,1)+1i*gj(:,2);  % return gamma to complex numbers
  figure(1); subplot(2,1,1); hold off;      % plot integrand
  dx = 0.003; c=1/(1+sqrt(max(E,0)));  % resolution, buffer for color plot
  xg = min(real(gj))-c:dx:max(real(gj))+c; yg = min(imag(gj))-c:dx:max(imag(gj))+c;
  %xg = -20:dx:20; yg = -3:dx:3; % for close radii
  [xx yy] = meshgrid(xg,yg); zz = xx + 1i*yy;
  a = norm(x-y)^2/4; b = (x(2)+y(2))/2 + E;     % params
  s0 = log(sqrt(2*(b+sqrt(b^2-a)*[-1 1]))); if imag(s0)==pi/2; s0=s0-1i*pi; end
  fprintf('saddle pts = %g+%gi, %g+%gi\n',real(s0(1)),imag(s0(1)),real(s0(2)),imag(s0(2)))
  t = exp(zz); p = a./t + b*t - t.^3/12; f = exp(1i*p); % integrand
  imagesc(xg,yg,real(f)); axis xy equal tight; caxis([-1 1]); colorbar;
  hold on; plot(g,'-','color',.7*[1 1 1]);  % contour
  plot(gj, 'k.','markersize',5); plot(real(s0),imag(s0),'w*');  % actual nodes
  subplot(2,1,2); semilogy(real(gj), sj, 'k.'); axis tight; hline(1e-16,'r-');
  xlabel('Re(s)'); ylabel('summand'); v=axis; v(3)=1e-20; axis(v); drawnow;
end
end %%%%%

if 0 % check convergence over a plane of target points...
E = 20;
y = [1;2];
gx = linspace(0,40,200); gy = linspace(-30,10,200);
[xx yy] = meshgrid(gx,gy); x = [xx(:)'; yy(:)']; N = size(x,2);
tic; u = lhelmfs(x,y,[],E); fprintf('#evals/sec (no derivs) = %.3g\n',N/toc)
u = reshape(u,size(xx)); figure(2); imagesc(gx,gy,real(u)); axis xy equal tight;
caxis(.07*[-1 1]);
o.meth=0; tic; v = lhelmfs(x,y,[],E,[],o);  % old method (default stdquad)
fprintf('#evals/sec (no derivs, meth=0) = %.3g\n',N/toc)
fprintf('sup over grid error nrm btw meth 0 and 1 = %.3g\n',max(abs(u(:)-v(:))));

tic; [u u1 u2] = lhelmfs(x,y,[],E); fprintf('#evals/sec (w/ derivs) = %.3g\n',N/toc)
o.meth=0; tic; [v v1 v2] = lhelmfs(x,y,[],E,[],o);  % old method (default stdquad)
fprintf('#evals/sec (w/ derivs, meth=0) = %.3g\n',N/toc)
fprintf('deriv sup over grid error nrm btw meth 0 and 1 = %.3g\n\n',...
        max(abs([u1(:)-v1(:); u2(:)-v2(:)])));

o.meth=1; o.h=.25; [v v1 v2] = lhelmfs(x,y,[],E,[],o); % slightly better node spacing
fprintf('sup over grid error nrm btw h=0.25 & h=default : val %.3g, grad %.3g\n',...
        max(abs(u(:)-v(:))), max(abs([u1(:)-v1(:); u2(:)-v2(:)])));
end
